/* 
 *  vcdpad pads VCD MPEGs to be usable by mkvcdfs
 *
 *  Copyright (C) 2000 Stefan Berndtsson <stefan@nocrew.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <stdlib.h>
#include <fcntl.h>

#define FSIZE 2324

int main(int argc, char *argv[])
{
  int fd,ofd;
  int off,block,sum=0;
  int i,oldi=16;
  char *input = "input.mpg";
  char *output = "output.mpg";
  static unsigned char buf[FSIZE*3];
  static unsigned char out[FSIZE*3];

  if(argc == 2) {
    input = argv[1];
  } else if(argc == 3) {
    input = argv[1];
    output = argv[2];
  } else {
    fprintf(stdout, "Usage: %s [input [output]]\n", argv[0]);
    exit(-1);
  }

  if((fd = open(input, O_RDONLY)) < 0) {
    perror(input);
    exit(-1);
  }
  
  if((ofd = open(output, O_WRONLY|O_CREAT|O_TRUNC, 00644)) < 0) {
    perror(output);
    exit(-1);
  }

  for(i=0;i<FSIZE;i++) {
    out[i] = 0;
  }

  while(read(fd, buf, FSIZE*2) == FSIZE*2) {
    sum += FSIZE;
    fprintf(stderr, "\rRead %dM", (int)sum/1048576);
    fflush(stderr);

    if((buf[0] == 0) &&
       (buf[1] == 0) &&
       (buf[2] == 1) &&
       (buf[3] == 0xba)) {
      block=FSIZE;
    } else {
      block=-1;
    }
    if(block != -1)
      for(i=1;i<FSIZE;i++) {
	if((buf[i] == 0) &&
	   (buf[i+1] == 0) &&
	   (buf[i+2] == 1) &&
	   (buf[i+3] == 0xba)) {
	  block = i;
	  break;
	}
      }
    
    if(block == FSIZE) {
      write(ofd, buf, FSIZE);
      lseek(fd, -FSIZE, SEEK_CUR);
    }
    if((block > 0) && (block != FSIZE)) {
      write(ofd, buf, block);
      write(ofd, out, FSIZE-block);
      lseek(fd, -(FSIZE+(FSIZE-block)), SEEK_CUR);
    }
  }
  fprintf(stderr, "\nDone padding.\n");
  close(fd);
  close(ofd);
}


